%% Ground Vibration Test Data and Analysis for the Body Freedom Flutter
% Identification of natural frequencies, damping ratios and mode shapes of
% the aircraft. In addition, a state-space model with 34 outputs is 
% obtained to represent the dynamic behavior of the aircraft.

clear all, close all
% Load data
% The units of the responses Accel/Force [g/lb]
load expdatashk.mat  % 34 FRFs obtained from swept sine excitation
sysh=frd(Hdata,omega);  % Create FRD object

% Plot experimental frequency response from point 12 to point 18 (tip of
% the wing) - Units [g/N]
[magnp,phasp,freqp]=bode(sysh(18,1)*4.45,{3*2*pi 35*2*pi});
subplot(211), plot(freqp/2/pi,mag2db(squeeze(magnp)),'LineWidth',2), grid, axis tight
title('Frequency response: From Force at point 12 to Accel at point 18 [g/N]')
ylabel('Magnitude (dB)')
subplot(212), plot(freqp/2/pi,(squeeze(phasp)),'LineWidth',2), grid, axis tight
xlabel('Frequency (Hz)')
ylabel('Phase (deg)')
set(gca,'YTick',[-900;-720;-540;-360;-180; 0])

%% Eigensystem Realization Algorithm (ERA)
% Using ERA to find natural frequencies for each frequency data collected.
% 34 state-space systems are obtained and their corresponding frequencies
% are plotted to estimate the observed modes.
% The algorithm is based on the Hankel matrix of the system. The Hankel
% matrix is constructed with the impulse response of the system.
% The impulse response of the system is not directly available but it can
% be computed using the inverse Fourier transform and the frequency
% response collected in the experiment. Columns of the Hankel matrix must
% be at least 10 times the number of modes to estimate and rows should be
% 3-5 times the columns. 

% Impulse response: The data is collected from 3 to 37 Hz. Hence, to apply
% inverse Fourier transform and obtain a real impulse response, the missing
% data from 0 to 3 Hz must be added.
Fs=74; Ts=1/Fs; %Sampling rate from dynamic analyzer
dFs = 34/400; % Sampling step 
Fmis = (0:dFs:3-dFs)*2*pi; % Frequency data missing
% Size of Hankel matrix
ncols=100; nrows=500; 
% Size of state-space model to estimate
inputs=1; n=14;
fq=[]; zq=[];
for i=1:34
    % Complete frequency response vector from 0 to 37 Hz
    Y1 = [ones(length(Fmis),1)*Hdata(i,:,1) ;squeeze(Hdata(i,:,:))];
    % Real impulse response
    y = ifft(ifftshift([flipud((Y1').'); Y1(2:end)]));
    % ERA algorithm
    [A,B,C,D]=era(squeeze(y),n,nrows,ncols,Ts,2);
    % Natural frequencies and damping ratios
    [wn,zn]=damp(A);
    [wn,idx] = sort(wn);
    zn = zn(idx);
    % Matrix array
    fq = [fq wn/2/pi];
    zq = [zq zn];
end

figure
plot(fq',[1:34]'*ones(1,n),'xm','LineWidth',2)
hold on

% Frequency average
idx1 = find(fq>4.5 & fq<6.5); idx2 = find(fq>7 & fq<9);
idx3 = find(fq>17 & fq<19); idx4 = find(fq>19 & fq<21);
idx5 = find(fq>22 & fq<24); idx6 = find(fq>27 & fq<29);
fqav = [sum(fq(idx1))/length(idx1) sum(fq(idx2))/length(idx2) sum(fq(idx3))/length(idx3)...
    sum(fq(idx4))/length(idx4) sum(fq(idx5))/length(idx5) sum(fq(idx6))/length(idx6)]';
for ik = 1:6
    plot([fqav(ik) fqav(ik)],[1 34],'--','Color',[0.5 0.5 0.5],'LineWidth',1)
end
title('Natural Frequencies Identified')
xlabel('Frequency (Hz)'), ylabel('Identified system: acceleration sensor #'),
xlim([3 35]), ylim([1 34])

%% MIMO state-space model estimation
% System Identification Toolbox. Subspace approach for initial estimate and
% minimization of the prediction error.

opts = ssestOptions('Display','on','Focus',[20,200]);
sysid = ssest(sysh,20,'Feedthrough',1,'Form','modal',opts);
save sysid.mat sysid
% load sysid.mat

% Plot comparison between estimated model and experimental data
figure
omega1=logspace(log10(3*2*pi),log10(35*2*pi),500);
[magnp,phasp,freqp]=bode(sysh(16,1)*4.45,omega1);
[magnd,phasd,freqd]=bode(sysid(16,1)*4.45,omega1);
subplot(211), plot(freqp/2/pi,mag2db(squeeze(magnp)),'--b',freqd/2/pi,mag2db(squeeze(magnd)),'-r','LineWidth',2), grid, axis tight
title('Frequency response: From Force at point 12 to Accel at point 16')
ylabel('Magnitude (dB)')
subplot(212), plot(freqp/2/pi,(squeeze(phasp))+360,'--b',freqd/2/pi,(squeeze(phasd)),'-r','LineWidth',2), grid, axis tight
xlabel('Frequency (Hz)')
ylabel('Phase (deg)')
legend('Experimental data','Estimated model (20-state)')
set(gca,'YTick',[0; 90;180;270;360])
xlabel('Frequency (Hz)')

figure
[magnp,phasp,freqp]=bode(sysh(20,1)*4.45,omega1);
[magnd,phasd,freqd]=bode(sysid(20,1)*4.45,omega1);
subplot(211), plot(freqp/2/pi,mag2db(squeeze(magnp)),'--b',freqd/2/pi,mag2db(squeeze(magnd)),'-r','LineWidth',2), grid, axis tight
title('Frequency response: From Force at point 12 to Accel at point 20')
ylabel('Magnitude (dB)')
subplot(212), plot(freqp/2/pi,(squeeze(phasp))+360,'--b',freqd/2/pi,(squeeze(phasd))-360,'-r','LineWidth',2), grid, axis tight
xlabel('Frequency (Hz)')
ylabel('Phase (deg)')
legend('Experimental data','Estimated model (20-state)')
set(gca,'YTick',[-360;-180; 0;180;360])

%% Validation:  measure of goodness of the fit
% Normalized root mean square (NRMSE) 
csysid = ss(sysid);
[~,fit,~]=compare(sysh,csysid);
idx1 = find(fit<70); idx2 = find(fit>70); 
figure, plot([70 70],[1 34],'--r',fit(idx1),idx1,'.r',fit(idx2),idx2,'.k','LineWidth',2)
xlabel('%fit = 100(1-NRMSE) (%)')
ylabel('Estimated system: acceleration sensor #')
legend('Threshold = 70%')
xlim([0 100]), ylim([1 34])

% Nu-Gap metric
[nugapvec] = freqgapmetric(sysh(idx2,1),csysid(idx2,1),omega);
omehz = omega/2/pi;
figure, plot([omehz(1) omehz(end)],[0.3 0.3],'--r',omehz,nugapvec,'k','LineWidth',2), grid
xlim([5 35]), ylim([0 1])
xlabel('Frequency (Hz)')
ylabel('Gap metric value')
legend('Threshold = 0.3')

%% Mode shapes
newcsys = modred(csysid,[1:2 13:14 17:20]);
[wn,z] = damp(newcsys);

for ii = 1:34
    fdata = squeeze(freqresp(csysid(ii,1),wn));
    imgp10(:,ii) = imag(fdata);
end

xcoor1 = [0 8 14.5 14.5+3.84 14.5+3.84*3 29.84+3.82 29.84+3*3.82 45.13+3.22 45.13+3*3.22];
xcoor = [-xcoor1 fliplr(-xcoor1) xcoor1(2:end) fliplr(xcoor1(2:end))];
ycoor1 = [0 14 0.373*(xcoor1(3:end)-14.5)+16.626];
chor = [34 17 12 12 12 12 12 12 12];
ycoor = [ycoor1 fliplr(ycoor1+chor) ycoor1(2:end)+chor(2:end) fliplr(ycoor1(2:end))];

% Sym Bending = 5.5 Hz
idx = 1;
zcoor = imgp10(idx,[1:9 fliplr(12:18) 11 10 27:34 fliplr(19:26)]);
figure
p=fill3(xcoor,ycoor,zcoor,'b');
set(p,'EdgeColor','b')
hold on
p=fill3(xcoor,ycoor,zeros(1,length(xcoor)),[0.9 0.9 0.9]);
set(p,'Marker','.','MarkerEdgeColor',[0.8 0.8 0.8],'EdgeColor',[0.8 0.8 0.8])
title(strcat('Sym 1st Bending =',num2str(wn(idx)/2/pi),'Hz'))
set(gca,'Visible','off'), axis tight
set(gca,'CameraPosition', [183.83 -191.3 430.659])

% A/S Bending = 8.556 Hz
idx = 3;
zcoor = imgp10(idx,[1:9 fliplr(12:18) 11 10 27:34 fliplr(19:26)]);
figure
p=fill3(xcoor,ycoor,zcoor,'g');
set(p,'EdgeColor','g')
hold on
p=fill3(xcoor,ycoor,zeros(1,length(xcoor)),[0.9 0.9 0.9]);
set(p,'Marker','.','MarkerEdgeColor',[0.8 0.8 0.8],'EdgeColor',[0.8 0.8 0.8])
title(strcat('A/S 1st Bending =',num2str(wn(idx)/2/pi),'Hz'))
set(gca,'Visible','off'), axis tight
set(gca,'CameraPosition', [168.1643 -334.1999  240.6367])

% Sym Torsion = 18.75 Hz
idx = 5;
zcoor = imgp10(idx,[1:9 fliplr(12:18) 11 10 27:34 fliplr(19:26)]);
% Point four is a bit off... Affecting the mode shape just for symmetric
% torsion
figure
p=fill3(xcoor,ycoor,zcoor,'r');
set(p,'EdgeColor','r')
hold on
p=fill3(xcoor,ycoor,zeros(1,length(xcoor)),[0.9 0.9 0.9]);
set(p,'Marker','.','MarkerEdgeColor',[0.8 0.8 0.8],'EdgeColor',[0.8 0.8 0.8])
title(strcat('Sym 1st Torsion =',num2str(wn(idx)/2/pi),'Hz'))
set(gca,'Visible','off'),axis tight
set(gca,'CameraPosition', [122.0823 -236.6912   20.9731])

% A/S Torsion = 19.770 Hz
idx = 7;
zcoor = imgp10(idx,[1:9 fliplr(12:18) 11 10 27:34 fliplr(19:26)]);
figure
p=fill3(xcoor,ycoor,zcoor,'y');
set(p,'EdgeColor','y')
hold on
p=fill3(xcoor,ycoor,zeros(1,length(xcoor)),[0.9 0.9 0.9]);
set(p,'Marker','.','MarkerEdgeColor',[0.8 0.8 0.8],'EdgeColor',[0.8 0.8 0.8])
title(strcat('A/S 1st Torsion =',num2str(wn(idx)/2/pi),'Hz'))
set(gca,'Visible','off'),axis tight
set(gca,'CameraPosition', [174.1095 -255.5131   51.4866])

% Sym 2nd Bending = 19.770 Hz
idx = 9;
zcoor = imgp10(idx,[1:9 fliplr(12:18) 11 10 27:34 fliplr(19:26)]);
figure
p=fill3(xcoor,ycoor,zcoor,'m');
set(p,'EdgeColor','m')
hold on
p=fill3(xcoor,ycoor,zeros(1,length(xcoor)),[0.9 0.9 0.9]);
set(p,'Marker','.','MarkerEdgeColor',[0.8 0.8 0.8],'EdgeColor',[0.8 0.8 0.8])
title(strcat('Sym 2nd Bending =',num2str(wn(idx)/2/pi),'Hz'))
set(gca,'CameraPosition', [162.3333 -256.7362   25.7433])
set(gca,'Visible','off'),axis tight

% A/S 2nd Bending = 28.730 Hz
idx = 11;
zcoor = imgp10(idx,[1:9 fliplr(12:18) 11 10 27:34 fliplr(19:26)]);
figure
p=fill3(xcoor,ycoor,zcoor,'k');
set(p,'EdgeColor','k')
hold on
p=fill3(xcoor,ycoor,zeros(1,length(xcoor)),[0.9 0.9 0.9]);
set(p,'Marker','.','MarkerEdgeColor',[0.8 0.8 0.8],'EdgeColor',[0.8 0.8 0.8])
title(strcat('A/S 2nd Bending =',num2str(wn(idx)/2/pi),'Hz'))
set(gca,'CameraPosition', [203.5161 -201.8036  143.5935])
set(gca,'Visible','off'),axis tight